selfDir_3d1bc249 := $(dir $(lastword $(MAKEFILE_LIST)))

# Make
subMake = $(MAKE) $(@F) --directory=$(@D)
+_ = $(+:_%=)
addBeforeExt = $(basename $(2))$(1)$(suffix $(2))

# Commands
CP = cp -p
DIFF = diff --unified=2

tablesSort := plots organisms stems
vegxMaps := $(wildcard $(tablesSort:%=map.VegX.%.csv))
tables := $(vegxMaps:map.VegX.%.csv=%)
vegbienMaps := $(subst .VegX.,.VegBIEN.,$(vegxMaps))

root := $(selfDir_3d1bc249)..
psqlAsBien := $(root)/bin/psql_vegbien

# Config
test_n ?= 2

#####

all: _always maps

.SUFFIXES:

_always:
.PHONY: _always

clean: _always
	$(RM) $(all)

%.out: % _always
	./$* >$@
.PRECIOUS: %.out

all += $(wildcard *.out)

$(root)/%: _always
	+$(subMake)

#####

reinstall: _always uninstall install

#####

maps: $(vegbienMaps) _always

all += $(vegbienMaps)

map.VegBIEN.%.csv: map.VegX.%.csv $(root)/mappings/VegX-VegBIEN.%.csv
	$(root)/bin/join_sort <$+ >$@
.PRECIOUS: map.VegBIEN.%.csv

#####

import: _always $(addprefix import-,$(tables)) verify

log = $*$(if $(n),.n=$(n),).log

logs := $(wildcard *.log)

rm_logs: _always
	$(RM) $(logs)

#####

verify: _always $(addprefix verify-,$(tables))

verify-%: verify.%.ref verify.%.out _always
	$(DIFF) $(+_)

%.out: %.sql _always
	$(psqlAsBien) --no-align --field-separator='	' --pset=footer=off <$< >$@
.PRECIOUS: %.out

#####

test: _always $(addprefix test-,$(tables))

test-%: test.%.out _always
	$(DIFF) $(<:.out=.ref) $<

accept-%: _always
	$(CP) $* $(*:.out=.ref)

#####

ifdef db

mapEnv := in_engine=$(dbEngine) in_database=$(db)

####

ifeq ($(dbEngine),MySQL)

bienPassword := $(shell cat $(root)/config/bien_password)
mysqlAs = mysql --user=$(1) --password='$(bienPassword)'
mysqlAsRoot := $(call mysqlAs,root)
mysqlAsBien := $(call mysqlAs,bien)

import-%: map.VegBIEN.%.csv _always
	-$(import_cmd)
import_cmd = (set -x; "time" env commit=1 $(map) $<) \
$(if $(n),,>>$(log)) 2>&1$(if $(n),|tee -a $(log))

%.ref: %.ref.sql
	$(mysqlAsBien) $(db) <$< >$@
.PRECIOUS: %.ref

###

install: _always db

uninstall: _always rm_db

db: src.$(db).sql _always
	-$(mysqlAsRoot) <$<
	echo "GRANT SELECT ON $(db).* TO 'bien'@'localhost';"|$(mysqlAsRoot)
# ignore errors in db import so that GRANT will still be run

rm_db: _always
	-echo "REVOKE ALL ON $(db).* FROM 'bien'@'localhost';"|$(mysqlAsRoot)
	echo "DROP DATABASE IF EXISTS $(db);"|$(mysqlAsRoot)
# ignore errors if grant not defined

###

else
$(error dbEngine variable must be one of the following: MySQL, PostgreSQL)
endif

####

test.%.out: map.VegBIEN.%.csv _always
	env test=1 n=$(test_n) $(map) $< >$@ 2>&1 || { e=$$?; cat $@; exit $$e;}
.PRECIOUS: test.%.out

#####

else
$(error db variable must be set)
endif

#####

# Must come after mapEnv is set
map := env $(mapEnv) out_database=vegbien $(root)/map
